
%               _       _   _ 
%    __  _____ | | ___ | |_| |
%    \ \/ / _ \| |/ _ \| __| |
%     >  < (_) | | (_) | |_| |
%    /_/\_\___/|_|\___/ \__|_|
%
% ### show
%
% **Syntax**
%
% ```matlab
% x.show('path/to/conductance/file')
% x.show(x.Compartment)
% x.show(x.Compartment.Conductance)
% x.show({conductance1, conductance2...})
% x.show(x.Compartment.Synapse)
% ```
%
% This method displays activation functions and timescales of any conductance or synapse. If you are 
% passing a conductance argument, subsequent calls to `show` will update the plot, 
% plotting the new activation curves over the old ones, allowing you to compare different channels.
%
%
%
% See Also: 
% xolotl.plot
% xolotl.getGatingFunctions





function show(self,obj, custom_name)

if isempty(self.Children)
	self.add('compartment','Default')
end
Ca_in = self.(self.child).Ca;
Ca_out = self.(self.child).Ca_out;


if isa(obj,'cpplab')
	% we have been given a cpplab object


	if strcmp(obj.cpp_class_name,'compartment')
		% we are being given a compartment, so we need to run this on every 
		% channel in that compartment
		channels = obj.find('conductance');
		for i = 1:length(channels)
			self.show(obj.(channels{i}),channels{i});
		end
		return
	end

	corelib.assert(strcmp(obj.cpp_class_parent,'conductance') || strcmp(obj.cpp_class_parent,'synapse'),'xolotl.show() only works for conductance-type or synapse-type objects')


elseif isa(obj,'char')
	obj = cpplab(obj);

elseif isa(obj,'cell')
	for i = 1:length(obj)
		xolotl.show(obj{i});
	end
	return
end

if strcmp(obj.cpp_class_name,'Leak')
	return
end



% specify range of V
V = linspace(-100,100,1e3);

% build a mex-able binary using the class path
if strcmp(obj.cpp_class_parent,'synapse')
	binary_name = str2func(xolotl.compileSynapseFcn(obj, self.cpp_folder));
	params = obj.get(obj.cpp_constructor_signature);

else
	binary_name = str2func(xolotl.compileActivationFcn(obj, self.cpp_folder));


	params = obj.get(obj.cpp_constructor_signature);

	[data.minf, data.hinf, data.taum, data.tauh] = binary_name(V, params, [self.temperature_ref; self.temperature; Ca_in; Ca_out]);

	if length(unique(data.hinf)) == 1
		data.tauh = NaN*data.tauh;
		data.hinf = NaN*data.tauh;
	end

end



if strcmp(obj.cpp_class_parent,'synapse')


	figure('outerposition',[300 300 1200 600],'PaperUnits','points','PaperSize',[1200 600],'Color','w'); hold on
	ax.sinf = subplot(1,2,1); hold on
	xlabel('V (mV)')
	ylabel('s_{\infty} ')

	ax.taus = subplot(1,2,2); hold on
	xlabel('V (mV)')
	ylabel('Approximate \tau_{s} (ms)')

	sinf = NaN*V;
	taus = NaN*V;

	[data.sinf, data.taus] = binary_name(V, params, [self.temperature_ref; self.temperature; Ca_in; Ca_out]);



	l = plot(ax.sinf,V,data.sinf,'k','DisplayName',obj.cpp_class_name);
	plot(ax.taus,V,data.taus,'k')

	legend(l);

	return

end




% check if there is any figure that we can plot on
allfigs = get(0, 'Children');
f = [];
for i = 1:length(allfigs)
	if ~isempty(allfigs(i).Tag) && strcmp(allfigs(i).Tag,'xolotl-show')
		f = allfigs(i);
		ax  = findall(f,'type','axes');

	end
end

if isempty(f)

	f = figure('outerposition',[100 100 1000 900],'PaperUnits','points','PaperSize',[1000 500],'Color','w'); hold on
	for i = 1:4
		ax(i) = subplot(2,2,i); hold on
		ax(i).FontSize = 16;
		ax(i).LineWidth = 1;
	end
	ax(1).Tag = 'minf';
	ax(2).Tag = 'hinf';
	ax(3).Tag = 'taum';
	ax(4).Tag = 'tauh';

	ylabel(ax(1),'m_{\infty}')
	xlabel(ax(1),'V (mV)')
	set(ax(1),'YLim',[-.01 1],'XLim',[-100 100])

	xlabel(ax(2),'V (mV)')
	ylabel(ax(2),'h_{\infty}')
	set(ax(2),'YLim',[-.01 1],'XLim',[-100 100])

	ylabel(ax(3),'\tau_{m} (ms)')
	xlabel(ax(3),'V (mV)')
	set(ax(3),'YScale','log','XLim',[-100 100])

	ylabel(ax(4),'\tau_{h} (ms)')
	xlabel(ax(4),'V (mV)')
	set(ax(4),'YScale','log','XLim',[-100 100])

	f.Tag = 'xolotl-show';

end



if nargin < 3
	custom_name = obj.cpp_class_name;
end


hash = hashlib.md5hash([obj.hash hashlib.md5hash([Ca_in,Ca_out])]);

% should we make a new plot, or just move the old one?
fn = fieldnames(data);
if isempty(ax(1).Children)
	plot(ax(find(strcmp({ax.Tag},'minf'))),V,data.minf,'DisplayName',custom_name,'Tag',hash,'LineWidth',2);
	plot(ax(find(strcmp({ax.Tag},'hinf'))),V,data.hinf,'DisplayName',custom_name,'Tag',hash,'LineWidth',2);
	plot(ax(find(strcmp({ax.Tag},'taum'))),V,data.taum,'DisplayName',custom_name,'Tag',hash,'LineWidth',2);
	plot(ax(find(strcmp({ax.Tag},'tauh'))),V,data.tauh,'DisplayName',custom_name,'Tag',hash,'LineWidth',2);



	% turn all YLim modes to auto
	for i = 3:length(ax)
		ax(i).YLimMode = 'auto';
	end


else
	% there are pre-existing plots. Make sure we don't plot the same object to two different plots
	for i = 1:length(ax)
		old_plots = ax(i).Children;
		plothere = find(strcmp({old_plots.Tag},hash));
		if isempty(plothere)
			% make new plot
			plot(ax(i),V,data.(fn{find(strcmp(fn,ax(i).Tag))}),'DisplayName',custom_name,'Tag',hash,'LineWidth',2);
		else
			% change YData in this plot
			plothere = ax(i).Children(plothere);
			plothere.YData = data.(fn{find(strcmp(fn,ax(i).Tag))});
		end
	end
end


axes(ax(1))
legend;
